/*
 * An implementation of Boyer-Moore matching for this problem: http://vn.spoj.com/problems/SUBSTR/
 * For more information about this implementation see  http://www.giaithuatlaptrinh.com/?p=245
 *  Created on: Jul 19, 2015
 *      Author: hunglv
 */
#include<stdio.h>
#include<iostream>
#include<stdlib.h>
#include<math.h>
#include<stack>
#include<string.h>

using namespace std;
#define max(x, y) (((x) < (y)) ? (y) : (x))
#define MAX 1000007
string T, P;
int  R[28];

/*
 * Zbar[i] is the lenght of the longest suffix of P[1,2,...,i]
 * matched with a suffix of P[1,2,..,m]
 */

int Zbar[MAX];
/*
 * LL[i] is the maximum index smaller than m such that P[i,i+1,..,m]
 * matched with a suffix of P[1,2,..,LL[i]] such that the char to the left of
 * of the matched substring of  P[1,2,..,LL[i]] is not equal from P[i-1]
 */
int LL[MAX];
/*
* SL[i] is the length of the longest suffix of P[i,i+1,..,m] that is matched with
* a prefix of P[1,2,...,m]
*/

int SL[MAX];

void fastfastBMmatcher(int n, int m);
void badCharBackup(int m);
void computeLL(int m);
void computeSL(int m);
void compute_Z_function(char *S,int *Z, int n);

int main() {
    cin >> T >> P;

    int n=T.size(), m=P.size();
    T = " " + T;
    P = " " + P ;
    fastfastBMmatcher(n,m);
}

/*
 * An implementation of Boyer-Moore with both bad character rule and good suffix rule
 */
void fastfastBMmatcher(int n, int m){
	badCharBackup(m);
	int i = 1, j = 0, p = 0;
	int  skip = 0, next = 0, stop = 1;
	computeLL(m);
	computeSL(m);
	while(i <= n-m+1){
		skip = 0;
		if(next == 1){ // just found a patern
			next = 0; // reset
		} else {
			stop = 1;
		}
		j = m;
		while(j >= stop && skip == 0){
			if(T[i+j-1] != P[j]){
				p = T[i+j-1]-96;
				skip = max(1, j - R[p]);
				if(j!=m){
					if(LL[j+1] !=0){
						skip = max(skip, m - LL[j+1]); // (1) and (2) of the good suffix rule
					} else {
						skip = max(skip, m - SL[j+1]); // (3) and (4) of the good suffix rule
					}
					/*
					 * For more information about the rules, see blogpost  http://www.giaithuatlaptrinh.com/?p=245
					 */
				}
			}
			j--;
		}
		if(skip == 0){
			printf("%d ", i);
			if(m > 1){
				skip = m - SL[2];
				stop = SL[2]+1;
				next = 	1; // find the next pattern;
			} else { // if m = 1, SL[1,2,...,m] is not defined
				skip = 1;
			}
		}
		i = i + skip;
	}
}

void badCharBackup(int m){
	int i = 0;
	memset(R, 0, sizeof(R));
	for(i = 1; i <= m; i++){
		R[P[i] - 96] = i;
	}
}

void computeLL(int m){
	int i = 0, j = 0;
	char Prev[m+1]; // the reverse of P[1,2,....,m]
	int Zrev[m+1]; // the Z function of Prev[1,2,...,m]
	for(i = m; i >= 1; i--){
		Prev[m-i+1] = P[i];
	}
	compute_Z_function(Prev, Zrev, m);
	for(i = 1; i<= m-1; i++){
		Zbar[i] = Zrev[m-i+1];//
	}
	memset(LL, 0, sizeof(LL));
	for(j = 1; j <= m-1; j++){
		i = m - Zbar[j]+1;
		LL[i] = j;
	}
}

void computeSL(int m){ // for more explnation, see   http://www.giaithuatlaptrinh.com/?p=245
	int i = m,j = 0;
	for(i = 1; i  <= m-1; i++){
		if(Zbar[i] == i){
			SL[m-i+1] = i;
			j = i;
		} else {
			SL[m-i+1] = j;
		}
	}
}

/*
 * For more information about the Z-funciton, see http://www.giaithuatlaptrinh.com/?p=250
 */
void compute_Z_function(char *S,int *Z, int n){
	Z[1] = 0;
	int R = 0, L = 0;
	int i = 0, j = 0, k = 0, next = 0;
	for(i = 2; i <=n; i++){
		next  = 0;
		if(i >= R){
			k = i; j = 1;
			next = 1; // continue comparing characters of S
		} else {
			int al = i - L + 1;
			int be = R - L + 1;
			if(i + Z[al] <= R){
				Z[i] = Z[al];
			} else{ // i + Z[al] -1 = R[i-1]
				k = R + 1;
				j = be - al + 2;
				next = 1; // Z[i] is longer than Z[al], continue comparing characters of S
			}

		}
		if(next == 1){
			while(k <= n && S[k] == S[j]){
				k++; j++;
			}
			Z[i] = j-1;
			L = i; R = L + Z[i] - 1;
		}
	}
}
